/***************************************************************************
 *   Copyright (C) 2006 by Sebastien Laout                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#include <kapplication.h>
#include <kaboutdata.h>
#include <kconfiggroup.h>
#include <kaction.h>
#include <kicon.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <kpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <kguiitem.h>
#include <QMenu>
#include <qtextedit.h>
#include <qlayout.h>
#include <qlabel.h>
#include <kdialog.h>
#include <qhttp.h>
#include <kurl.h>
#include <kinputdialog.h>
#include <qvalidator.h>
#include <qaction.h>
#include <kdebug.h>
#include <kconfig.h>

#include <pwd.h>

#include <iostream>
#include <kglobal.h>

#include "likeback.h"
//#include "likeback_private.h"
#include <QIcon>
#include <kcmdlineargs.h>
#include <QDesktopWidget>
#include <QGroupBox>
#include <kactioncollection.h>
#include <ksharedconfig.h>
/****************************************/
/********** class LikeBackBar: **********/
/****************************************/

LikeBackBar::LikeBackBar ( LikeBack *likeBack )
		: QWidget ( 0, Qt::X11BypassWindowManagerHint | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint )
		, m_likeBack ( likeBack )
{
	setObjectName ( "LikeBackBar" );
	QHBoxLayout *layout = new QHBoxLayout ( this );

	QIcon likeIconSet    = KIcon ( "likeback_like" );
	QIcon dislikeIconSet = KIcon ( "likeback_dislike" );
	QIcon bugIconSet     = KIcon ( "likeback_bug" );
	QIcon featureIconSet = KIcon ( "likeback_feature" );

	m_likeButton = new QToolButton ( this );
	m_likeButton->setObjectName ( "likeback_like" );
	m_likeButton->setIcon ( likeIconSet );
	m_likeButton->setText ( "<p>" + i18n ( "Send KAlarm developers a comment about something you like" ) );    //DJ
	m_likeButton->setAutoRaise ( true );
	connect ( m_likeButton, SIGNAL ( clicked() ), this, SLOT ( clickedLike() ) );
	layout->addWidget ( m_likeButton );

	m_dislikeButton = new QToolButton ( this );
	m_dislikeButton->setObjectName ( "likeback_dislike" );
	m_dislikeButton->setIcon ( dislikeIconSet );
	m_dislikeButton->setText ( "<p>" + i18n ( "Send KAlarm developers a comment about something you dislike" ) );    //DJ
	m_dislikeButton->setAutoRaise ( true );
	connect ( m_dislikeButton, SIGNAL ( clicked() ), this, SLOT ( clickedDislike() ) );
	layout->addWidget ( m_dislikeButton );

	m_bugButton = new QToolButton ( this );
	m_bugButton->setObjectName ( "likeback_bug" );
	m_bugButton->setIcon ( bugIconSet );
	m_bugButton->setText ( "<p>" + i18n ( "Send KAlarm developers a comment about an improper behavior of KAlarm" ) );    //DJ
	m_bugButton->setAutoRaise ( true );
	connect ( m_bugButton, SIGNAL ( clicked() ), this, SLOT ( clickedBug() ) );
	layout->addWidget ( m_bugButton );

	m_featureButton = new QToolButton ( this );
	m_featureButton->setObjectName ( "likeback_feature" );
	m_featureButton->setIcon ( featureIconSet );
	m_featureButton->setText ( "<p>" + i18n ( "Send KAlarm developers a comment about a new feature you desire" ) );    //DJ
	m_featureButton->setAutoRaise ( true );
	connect ( m_featureButton, SIGNAL ( clicked() ), this, SLOT ( clickedFeature() ) );
	layout->addWidget ( m_featureButton );

	connect ( &m_timer, SIGNAL ( timeout() ), this, SLOT ( autoMove() ) );

	LikeBack::Button buttons = likeBack->buttons();
	m_likeButton->setShown ( buttons & LikeBack::Like );
	m_dislikeButton->setShown ( buttons & LikeBack::Dislike );
	m_bugButton->setShown ( buttons & LikeBack::Bug );
	m_featureButton->setShown ( buttons & LikeBack::Feature );
}

LikeBackBar::~LikeBackBar()
{
}

void LikeBackBar::startTimer()
{
	m_timer.start ( 10 );
}

void LikeBackBar::stopTimer()
{
	m_timer.stop();
}

void LikeBackBar::autoMove()
{
	static QWidget *lastWindow = 0;

	QWidget *window = kapp->activeWindow();
	// When a Kicker applet has the focus, like the Commandline QLineEdit,
	// the systemtray icon indicates to be the current window and the LikeBack is shown next to the system tray icon.
	// It's obviously bad ;-) :
	bool shouldShow = ( m_likeBack->userWantsToShowBar() && m_likeBack->enabledBar() && window && !window->inherits ( "KSystemTray" ) );
	//DJ start
	if (shouldShow && window)
	{
		if (!m_likeBack->disabledClasses().isEmpty())
		{
#warning next line
			QString win = QString::fromLatin1(window->metaObject()->className());
			QList<QRegExp> classes = m_likeBack->disabledClasses();
			for (int i = 0, count = classes.count();  i < count;  ++i)
			{
				if (win.indexOf(classes[i]) >= 0)
				{
					shouldShow = false;
					break;
				}
			}
		}
		if (!m_likeBack->enabledWindows().isEmpty() && (!shouldShow || m_likeBack->disabledClasses().isEmpty()))
		{
			// Override any disabled classes
			shouldShow = false;
#warning next line
			QString win = window->objectName();
			QList<QRegExp> windows = m_likeBack->enabledWindows();
			for (int i = 0, count = windows.count();  i < count;  ++i)
			{
				if (win.indexOf(windows[i]) >= 0)
				{
					shouldShow = true;
					break;
				}
			}
		}
	}
	//DJ end
	if ( shouldShow )
	{
		//move(window->x() + window->width() - 100 - width(), window->y());
		//move(window->x() + window->width() - 100 - width(), window->mapToGlobal(QPoint(0, 0)).y() - height());
		move ( window->mapToGlobal ( QPoint ( 0, 0 ) ).x() + window->width() - width(), window->mapToGlobal ( QPoint ( 0, 0 ) ).y() + 1 );
		QString debugStr;

		if ( window != lastWindow && m_likeBack->windowNamesListing() != LikeBack::NoListing )
		{
			if ( qstricmp ( window->objectName().toLocal8Bit().data(), "" ) == 0 || qstricmp ( window->objectName().toLocal8Bit().data(), "unnamed" ) == 0 )
			{
				QTextStream ( &debugStr ) << "===== LikeBack ===== UNNAMED ACTIVE WINDOW OF TYPE " << window->metaObject()->className() << " ======" << LikeBack::activeWindowPath();
				qDebug() <<debugStr;
			}
			else if ( m_likeBack->windowNamesListing() == LikeBack::AllWindows )
			{
				QTextStream ( &debugStr ) << "LikeBack: Active Window: " << LikeBack::activeWindowPath();
				qDebug() <<debugStr;
			}
		}
		lastWindow = window;
	}

	// Show or hide the bar accordingly:
	if ( shouldShow && !isVisible() )
	{
		show();
	}
	else if ( !shouldShow && isVisible() )
	{
		hide();
	}
}

void LikeBackBar::clickedLike()
{
	m_likeBack->execCommentDialog ( LikeBack::Like );
}

void LikeBackBar::clickedDislike()
{
	m_likeBack->execCommentDialog ( LikeBack::Dislike );
}

void LikeBackBar::clickedBug()
{
	m_likeBack->execCommentDialog ( LikeBack::Bug );
}

void LikeBackBar::clickedFeature()
{
	m_likeBack->execCommentDialog ( LikeBack::Feature );
}

/********************************************/
/********** class LikeBackPrivate: **********/
/********************************************/

LikeBackPrivate::LikeBackPrivate()
		: bar ( 0 )
		, config(0)
		, aboutData ( 0 )
		, buttons ( LikeBack::DefaultButtons )
		, hostName()
		, remotePath()
		, hostPort ( 80 )
		, acceptedLocales()
		, acceptedLanguagesMessage()
		, windowListing ( LikeBack::NoListing )
		, showBar ( false )
		, disabledCount ( 0 )
		, fetchedEmail()
		, action ( 0 )
{
}

LikeBackPrivate::~LikeBackPrivate()
{
	delete bar;
	delete action;
	delete config;

	aboutData = 0;
}

/*************************************/
/********** class LikeBack: **********/
/*************************************/

LikeBack::LikeBack ( Button buttons, bool showBarByDefault, KConfig* config, const KAboutData *aboutData )
		: QObject()
{
	// Initialize properties (1/2):
	d = new LikeBackPrivate();
	d->buttons          = buttons;
	if (config)
		d->config = new KConfigGroup(config, "LikeBack");
	else
		d->config = new KConfigGroup(KGlobal::config(), "LikeBack");
	d->aboutData        = aboutData;
	d->showBarByDefault = showBarByDefault;

	// Use default KApplication config and aboutData if not provided:
	if ( d->aboutData == 0 )
		d->aboutData = KCmdLineArgs::aboutData( );

	// Initialize properties (2/2) [Needs aboutData to be set]:
	d->showBar          = userWantsToShowBar();

	// Fetch the KControl user email address as a default one:
	if ( !emailAddressAlreadyProvided() )
		fetchUserEmail();

	// Initialize the button-bar:
	d->bar = new LikeBackBar ( this );
	d->bar->resize ( d->bar->sizeHint() );

	// Show the information message if it is the first time, and if the button-bar is shown:
	static const char *messageShown = "LikeBack_starting_information";
	if ( d->showBar && KMessageBox::shouldBeShownContinue ( messageShown ) )
	{
		showInformationMessage();
		KMessageBox::saveDontShowAgainContinue ( messageShown );
	}

	// Show the bar if that's wanted by the developer or the user:
	if ( d->showBar )
		QTimer::singleShot ( 0, d->bar, SLOT ( startTimer() ) );

#if 0
	disableBar();
	// Alex: Oh, it drove me nuts
	d->buttons = ( Button ) ( 0 ); showInformationMessage();
	d->buttons = ( Button ) ( Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Bug ); showInformationMessage();
	d->buttons = ( Button ) ( Bug | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Dislike ); showInformationMessage();
	d->buttons = ( Button ) ( Dislike       | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Dislike | Bug ); showInformationMessage();
	d->buttons = ( Button ) ( Dislike | Bug | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Like ); showInformationMessage();
	d->buttons = ( Button ) ( Like                 | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Like           | Bug ); showInformationMessage();
	d->buttons = ( Button ) ( Like           | Bug | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Like | Dislike ); showInformationMessage();
	d->buttons = ( Button ) ( Like | Dislike       | Feature ); showInformationMessage();
	d->buttons = ( Button ) ( Like | Dislike | Bug ); showInformationMessage();
	d->buttons = ( Button ) ( Like | Dislike | Bug | Feature ); showInformationMessage();
	enableBar();
#endif
}

LikeBack::~LikeBack()
{
	delete d;
}

void LikeBack::setWindowNamesListing ( WindowListing windowListing )
{
	d->windowListing = windowListing;
}

LikeBack::WindowListing LikeBack::windowNamesListing()
{
	return d->windowListing;
}

void LikeBack::setAcceptedLanguages ( const QStringList &locales, const QString &message )
{
	d->acceptedLocales          = locales;
	d->acceptedLanguagesMessage = message;
}

QStringList LikeBack::acceptedLocales()
{
	return d->acceptedLocales;
}

QString LikeBack::acceptedLanguagesMessage()
{
	return d->acceptedLanguagesMessage;
}

void LikeBack::setServer ( const QString &hostName, const QString &remotePath, quint16 hostPort )
{
	d->hostName   = hostName;
	d->remotePath = remotePath;
	d->hostPort   = hostPort;
}

QString LikeBack::hostName()
{
	return d->hostName;
}

QString LikeBack::remotePath()
{
	return d->remotePath;
}

quint16 LikeBack::hostPort()
{
	return d->hostPort;
}

//DJ start
void LikeBack::setSubVersion(const QString &subVersion)
{
	d->subVersion = subVersion;
}

QString LikeBack::subVersion()
{
	return d->subVersion;
}

void LikeBack::setDisabledClasses(const QList<QByteArray>& classNames)
{
	d->disabledClasses.clear();
	for (int i = 0, count = classNames.count();  i < count;  ++i)
	{
		QByteArray item = classNames[i];
		if (item.length() > 0)
		{
			if (item[0] != '^')
				item.prepend("^");
			if (item[item.length()-1] != '$')
				item.append("$");
			d->disabledClasses += QRegExp(item);
		}
	}
}

const QList<QRegExp> &LikeBack::disabledClasses() const
{
	return d->disabledClasses;
}

void LikeBack::setEnabledWindows(const QStringList& wins)
{
	d->enabledWindows.clear();
	for (int i = 0, count = wins.count();  i < count;  ++i)
	{
		QString item = wins[i];
		if (item.length() > 0)
		{
			if (item[0] != '^')
				item.prepend("^");
			if (item[item.length()-1] != '$')
				item.append("$");
			d->enabledWindows += QRegExp(item);
		}
	}
}

const QList<QRegExp> &LikeBack::enabledWindows() const
{
	return d->enabledWindows;
}
//DJ end

void LikeBack::disableBar()
{
	d->disabledCount++;
	if ( d->bar && d->disabledCount > 0 )
	{
		d->bar->hide();
		d->bar->stopTimer();
	}
}

void LikeBack::enableBar()
{
	d->disabledCount--;
	if ( d->disabledCount < 0 )
		std::cerr << "===== LikeBack ===== Enabled more times than it was disabled. Please refer to the disableBar() documentation for more information and hints." << std::endl;
	if ( d->bar && d->disabledCount <= 0 )
	{
		d->bar->startTimer();
	}
}

bool LikeBack::enabledBar()
{
	return d->disabledCount <= 0;
}

void LikeBack::execCommentDialog ( Button type, const QString &initialComment, const QString &windowPath, const QString &context )
{
	disableBar();
	LikeBackDialog dialog ( type, initialComment, windowPath, context, this );
	dialog.exec();
	enableBar();
}

void LikeBack::execCommentDialogFromHelp()
{
	execCommentDialog ( AllButtons, /*initialComment=*/"", /*windowPath=*/"HelpMenuAction" );
}

LikeBack::Button LikeBack::buttons()
{
	return d->buttons;
}

const KAboutData* LikeBack::aboutData()
{
	return d->aboutData;
}

KConfigGroup* LikeBack::config()
{
	return d->config;
}

KAction* LikeBack::sendACommentAction ( KActionCollection *parent )
{
	if ( d->action == 0 )
	{
		d->action = new KAction ( this );
		d->action->setText ( i18n ( "&Send a Comment to Developers" ) );
		d->action->setIcon ( KIcon ( "mail_new" ) );
		if ( parent!=0 )
			parent-> addAction ( "likeback_send_a_comment", d->action );
	}
	d->action->setShortcut ( QString() );
	connect ( d->action, SIGNAL ( triggered ( bool ) ), SLOT ( execCommentDialog() ) );
	return d->action;
}

bool LikeBack::userWantsToShowBar()
{
	// Store the button-bar per version, so it can be disabled by the developer for the final version:
	return d->config->readEntry ( "userWantToShowBarForVersion_" + d->aboutData->version(), d->showBarByDefault );
}

void LikeBack::setUserWantsToShowBar ( bool showBar )
{
	if ( showBar == d->showBar )
		return;

	d->showBar = showBar;

	// Store the button-bar per version, so it can be disabled by the developer for the final version:
	d->config->writeEntry ( "userWantToShowBarForVersion_" + d->aboutData->version(), showBar );
	d->config->sync(); // Make sure the option is saved, even if the application crashes after that.

	if ( showBar )
		d->bar->startTimer();
}

void LikeBack::showInformationMessage()
{
	// Load and register the images needed by the message:
	QPixmap likeIcon    = KIcon ( "likeback_like" ).pixmap(16,16);
	QPixmap dislikeIcon = KIcon ( "likeback_dislike" ).pixmap(16,16);
	QPixmap bugIcon     = KIcon ( "likeback_bug" ).pixmap(16,16);
	QPixmap featureIcon = KIcon ( "likeback_feature" ).pixmap(16,16);
/* FIXME 1.5	QMimeSourceFactory::defaultFactory()->setPixmap ( "likeback_icon_like",    likeIcon );
	QMimeSourceFactory::defaultFactory()->setPixmap ( "likeback_icon_dislike", dislikeIcon );
	QMimeSourceFactory::defaultFactory()->setPixmap ( "likeback_icon_bug",     bugIcon );
	QMimeSourceFactory::defaultFactory()->setPixmap ( "likeback_icon_feature", featureIcon );
*/
	// Show a message reflecting the allowed types of comment:
	Button buttons = d->buttons;
	int nbButtons = ( buttons & Like    ? 1 : 0 ) +
	                ( buttons & Dislike ? 1 : 0 ) +
	                ( buttons & Bug     ? 1 : 0 ) +
	                ( buttons & Feature ? 1 : 0 );
	KMessageBox::information ( 0,
	                           "<p><b>" + ( isDevelopmentVersion ( d->aboutData->version() ) ?
	                                        i18n ( "Welcome to this testing version of %1.", d->aboutData->programName() ) :
	                                        i18n ( "Welcome to %1.", d->aboutData->programName() ) ) +
				   "</b></p>"
	                           "<p>" + i18n ( "To help us improve it, your comments are important." ) + "</p>"
	                           "<p>" +
	                           ( ( buttons & LikeBack::Like ) && ( buttons & LikeBack::Dislike ) ?
	                             i18n ( "Each time you have a great or frustrating experience, "
	                                    "please click the appropriate face below the window title-bar, "
	                                    "briefly describe what you like or dislike and click Send." )
	                             : ( buttons & LikeBack::Like ?
	                                 i18n ( "Each time you have a great experience, "
	                                        "please click the smiling face below the window title-bar, "
	                                        "briefly describe what you like and click Send." )
	                                 : ( buttons & LikeBack::Dislike ?
	                                     i18n ( "Each time you have a frustrating experience, "
	                                            "please click the frowning face below the window title-bar, "
	                                            "briefly describe what you dislike and click Send." )
	                                     :
	                                     QString()
	                                   ) ) ) + "</p>" +
	                           ( buttons & LikeBack::Bug ?
	                             "<p>" +
	                             ( buttons & ( LikeBack::Like | LikeBack::Dislike ) ?
	                               i18n ( "Follow the same principle to quickly report a bug: "
	                                      "just click the broken-object icon in the top-right corner of the window, describe it and click Send." )
	                               :
	                               i18n ( "Each time you discover a bug in KAlarm, "    //DJ
	                                      "please click the broken-object icon below the window title-bar, "
	                                      "briefly describe the mis-behaviour and click Send." )
	                             ) + "</p>"
                             : "" ) +
	                           "<p>" + i18nc ( "Example:", "Examples:", nbButtons ) + "</p>" +
	                           ( buttons & LikeBack::Like ?
	                             "<p><img source=\"likeback_icon_like\"> &nbsp;" +
	                             i18n ( "<b>I like</b> the new artwork. Very refreshing." ) + "</p>"
	                             : "" ) +
	                           ( buttons & LikeBack::Dislike ?
	                             "<p><img source=\"likeback_icon_dislike\"> &nbsp;" +
	                             i18n ( "<b>I dislike</b> the welcome page of that assistant. Too time consuming." ) + "</p>"
	                             : "" ) +
	                           ( buttons & LikeBack::Bug ?
	                             "<p><img source=\"likeback_icon_bug\"> &nbsp;" +
	                             i18n ( "<b>KAlarm has an improper behaviour</b> when clicking the Add button. Nothing happens." ) + "</p>"    //DJ
	                             : "" ) +
	                           ( buttons & LikeBack::Feature ?
	                             "<p><img source=\"likeback_icon_feature\"> &nbsp;" +
	                             i18n ( "<b>I desire a new feature</b> allowing me to send my work by email." ) + "</p>"
	                             : "" ) +
	                           "</tr></table>",
	                           i18n ( "Help Improve KAlarm" ) );    //DJ

	// Reset the images from the factory:
/* FIXME 1.5	QMimeSourceFactory::defaultFactory()->setData ( "likeback_icon_like",    0L );
	QMimeSourceFactory::defaultFactory()->setData ( "likeback_icon_dislike", 0L );
	QMimeSourceFactory::defaultFactory()->setData ( "likeback_icon_bug",     0L );
	QMimeSourceFactory::defaultFactory()->setData ( "likeback_icon_feature", 0L );
*/
}

QString LikeBack::activeWindowPath()
{
	// Compute the window hierarchy (from the latest to the oldest):
	QStringList windowNames;
	QWidget *window = kapp->activeWindow();
	while ( window )
	{
		QString name = window->objectName();
		// Append the class name to the window name if it is unnamed:
		if ( name == "unnamed" )
			name += QString ( ":" ) + window->metaObject()->className();
		windowNames.append ( name );
		window = qobject_cast<QWidget*> ( window->parent() );
	}

	// Create the string of windows starting by the end (from the oldest to the latest):
	QString windowPath;
	for ( int i = ( ( int ) windowNames.count() ) - 1; i >= 0; i-- )
	{
		if ( windowPath.isEmpty() )
			windowPath = windowNames[i];
		else
			windowPath += QString ( "~~" ) + windowNames[i];
	}

	// Finally return the computed path:
	return windowPath;
}

bool LikeBack::emailAddressAlreadyProvided()
{
	return d->config->readEntry ( "emailAlreadyAsked", false );
}

QString LikeBack::emailAddress()
{
	if ( !emailAddressAlreadyProvided() )
		askEmailAddress();

	return d->config->readEntry ( "emailAddress", "" );
}

void LikeBack::setEmailAddress ( const QString &address, bool userProvided )
{
	d->config->writeEntry ( "emailAddress",      address );
	d->config->writeEntry ( "emailAlreadyAsked", userProvided || emailAddressAlreadyProvided() );
	d->config->sync(); // Make sure the option is saved, even if the application crashes after that.
}

void LikeBack::askEmailAddress()
{
	QString currentEmailAddress = d->config->readEntry ( "emailAddress", "" );
	if ( !emailAddressAlreadyProvided() && !d->fetchedEmail.isEmpty() )
		currentEmailAddress = d->fetchedEmail;

	bool ok;

	QString emailExpString = "[\\w-\\.]+@[\\w-\\.]+\\.[\\w]+";
	//QString namedEmailExpString = "[.]*[ \\t]+<" + emailExpString + '>';
	//QRegExp emailExp("^(|" + emailExpString + '|' + namedEmailExpString + ")$");
	QRegExp emailExp ( "^(|" + emailExpString + ")$" );
	QRegExpValidator emailValidator ( emailExp, this );

	disableBar();
	QString email = KInputDialog::getText (
	                    i18n ( "Email Address" ),
	                    "<p><b>" + i18n ( "Please provide your email address." ) + "</b></p>" +
	                    "<p>" + i18n ( "It will only be used to contact you back if more information is needed about your comments, ask you how to reproduce the bugs you report, send bug corrections for you to test, etc." ) + "</p>" +
	                    "<p>" + i18n ( "The email address is optional. If you do not provide any, your comments will be sent anonymously." ) + "</p>",
	                    currentEmailAddress, &ok, kapp->activeWindow(), &emailValidator );
	enableBar();

	if ( ok )
		setEmailAddress ( email );
}

// FIXME: Should be moved to KAboutData? Cigogne will also need it.
bool LikeBack::isDevelopmentVersion ( const QString &version )
{
	return version.indexOf ( "alpha", /*index=*/0, /*caseSensitive=*/Qt::CaseInsensitive ) != -1 ||
	       version.indexOf ( "beta",  /*index=*/0, /*caseSensitive=*/Qt::CaseInsensitive ) != -1 ||
	       version.indexOf ( "rc",    /*index=*/0, /*caseSensitive=*/Qt::CaseInsensitive ) != -1 ||
	       version.indexOf ( "svn",   /*index=*/0, /*caseSensitive=*/Qt::CaseInsensitive ) != -1 ||
	       version.indexOf ( "cvs",   /*index=*/0, /*caseSensitive=*/Qt::CaseInsensitive ) != -1;
}

/**
 * Code from KBugReport::slotConfigureEmail() in kdeui/kbugreport.cpp:
 */
/*void LikeBack::beginFetchingEmail()
{
	if (m_process)
		return;
	m_process = new K3Process();
	*m_process << QLatin1String("kcmshell") << QLatin1String("kcm_useraccount");
	connect( m_process, SIGNAL(processExited(K3Process*)), SLOT(fetchUserEmail()) );
	if (!m_process->start()) {
		kDebug() << "Couldn't start kcmshell.." << endl;
		delete m_process;
		m_process = 0;
		return;
	}
//	m_configureEmail->setEnabled(false);
}*/

/**
 * Code from KBugReport::slotSetFrom() in kdeui/kbugreport.cpp:
 */
//TODO
void LikeBack::fetchUserEmail()
{
////	delete m_process;
////	m_process = 0;
////	m_configureEmail->setEnabled(true);
//
//	// ### KDE4: why oh why is KEmailSettings in kio?
//	KConfig emailConf ( QString::fromLatin1 ( "emaildefaults" ) );
//
//	// find out the default profile
//	emailConf.setGroup ( QString::fromLatin1 ( "Defaults" ) );
//	QString profile = QString::fromLatin1 ( "PROFILE_" );
//	profile += emailConf.readEntry ( QString::fromLatin1 ( "Profile" ), QString::fromLatin1 ( "Default" ) );
//
//	emailConf.setGroup ( profile );
//	QString fromaddr = emailConf.readEntry ( QString::fromLatin1 ( "EmailAddress" ),QString() );
//	if ( fromaddr.isEmpty() )
//	{
//		struct passwd *p;
//		p = getpwuid ( getuid() );
//		d->fetchedEmail = QString::fromLatin1 ( p->pw_name );
//	}
//	else
//	{
//		QString name = emailConf.readEntry ( QString::fromLatin1 ( "FullName" ), QString() );
//		if ( !name.isEmpty() )
//			d->fetchedEmail = /*name + QString::fromLatin1(" <") +*/ fromaddr /*+ QString::fromLatin1(">")*/;
//	}
//	m_from->setText( fromaddr );
}

/*******************************************/
/********** class LikeBackDialog: **********/
/*******************************************/

LikeBackDialog::LikeBackDialog ( LikeBack::Button reason, const QString &initialComment, const QString &windowPath, const QString &context, LikeBack *likeBack )
	: KDialog ( kapp->activeWindow() )
	, m_likeBack ( likeBack )
	, m_windowPath ( windowPath )
	, m_context ( context )
	, m_originalShow(m_likeBack->userWantsToShowBar())   //DJ
{
	setObjectName ( "_likeback_feedback_window_" );
	setCaption ( i18n ( "Send a Comment to Developers" ) );
	setModal ( true );
	setButtons ( KDialog::Ok | KDialog::Cancel | KDialog::Default );
	setDefaultButton ( KDialog::Ok );
	showButtonSeparator ( true );
	// If no specific "reason" is provided, choose the first one:
	if ( reason == LikeBack::AllButtons )
	{
		LikeBack::Button buttons = m_likeBack->buttons();
		int firstButton = 0;
		if ( firstButton == 0 && ( buttons & LikeBack::Like ) )    firstButton = LikeBack::Like;
		if ( firstButton == 0 && ( buttons & LikeBack::Dislike ) ) firstButton = LikeBack::Dislike;
		if ( firstButton == 0 && ( buttons & LikeBack::Bug ) )     firstButton = LikeBack::Bug;
		if ( firstButton == 0 && ( buttons & LikeBack::Feature ) ) firstButton = LikeBack::Feature;
		reason = ( LikeBack::Button ) firstButton;
	}

	// If no window path is provided, get the current active window path:
	if ( m_windowPath.isEmpty() )
		m_windowPath = LikeBack::activeWindowPath();

	QWidget *page = new QWidget ( this );
	QVBoxLayout *pageLayout = new QVBoxLayout ( page );
	pageLayout->setMargin ( 0 );
	pageLayout->setSpacing ( spacingHint() );

	// The introduction message:
	QLabel *introduction = new QLabel ( introductionText(), page );
	pageLayout->addWidget ( introduction );

	// The comment group:
	m_group = new QButtonGroup ( 0 );//i18n("Send KAlarm Developers a Comment About:"), page);    //DJ
	QGroupBox *box = new QGroupBox ( i18n ( "Send KAlarm Developers a Comment About:" ), page );    //DJ
	QVBoxLayout *boxLayout = new QVBoxLayout ();
	box->setLayout ( boxLayout );
	pageLayout->addWidget ( box );

	// The radio buttons:
	QWidget *buttons = new QWidget ( box );
	boxLayout->addWidget(buttons);
	QGridLayout *buttonsGrid = new QGridLayout ( buttons );
	buttonsGrid->setMargin ( 0 );
	buttonsGrid->setSpacing ( spacingHint() );
	if ( m_likeBack->buttons() & LikeBack::Like )
	{
		QPixmap likePixmap = KIcon ( "likeback_like" ).pixmap ( 16, 16 );
		QLabel *likeIcon = new QLabel ( buttons );
		likeIcon->setPixmap ( likePixmap );
		likeIcon->setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
		QRadioButton *likeButton = new QRadioButton ( i18n ( "Something you &like" ), buttons );
		buttonsGrid->addWidget ( likeIcon,   /*row=*/0, /*column=*/0 );
		buttonsGrid->addWidget ( likeButton, /*row=*/0, /*column=*/1 );
		m_group->addButton ( likeButton, LikeBack::Like );
	}
	if ( m_likeBack->buttons() & LikeBack::Dislike )
	{
		QPixmap dislikePixmap = KIcon ( "likeback_dislike" ).pixmap ( 16, 16 );
		QLabel *dislikeIcon = new QLabel ( buttons );
		dislikeIcon->setPixmap ( dislikePixmap );
		dislikeIcon->setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
		QRadioButton *dislikeButton = new QRadioButton ( i18n ( "Something you &dislike" ), buttons );
		buttonsGrid->addWidget ( dislikeIcon,   /*row=*/1, /*column=*/0 );
		buttonsGrid->addWidget ( dislikeButton, /*row=*/1, /*column=*/1 );
		m_group->addButton ( dislikeButton, LikeBack::Dislike );
	}
	if ( m_likeBack->buttons() & LikeBack::Bug )
	{
		QPixmap bugPixmap = KIcon ( "likeback_bug" ).pixmap ( 16,16 );
		QLabel *bugIcon = new QLabel ( buttons );
		bugIcon->setPixmap ( bugPixmap );
		bugIcon->setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
		QRadioButton *bugButton = new QRadioButton ( i18n ( "An improper &behavior of KAlarm" ), buttons );    //DJ
		buttonsGrid->addWidget ( bugIcon,   /*row=*/2, /*column=*/0 );
		buttonsGrid->addWidget ( bugButton, /*row=*/2, /*column=*/1 );
		m_group->addButton ( bugButton, LikeBack::Bug );
	}
	if ( m_likeBack->buttons() & LikeBack::Feature )
	{
		QPixmap featurePixmap = KIcon ( "likeback_feature" ).pixmap ( 16,16 );
		QLabel *featureIcon = new QLabel ( buttons );
		featureIcon->setPixmap ( featurePixmap );
		featureIcon->setSizePolicy ( QSizePolicy::Fixed, QSizePolicy::Fixed );
		QRadioButton *featureButton = new QRadioButton ( i18n ( "A new &feature you desire" ), buttons );
		buttonsGrid->addWidget ( featureIcon,   /*row=*/3, /*column=*/0 );
		buttonsGrid->addWidget ( featureButton, /*row=*/3, /*column=*/1 );
		m_group->addButton ( featureButton, LikeBack::Feature );
	}
	m_group->button ( reason )->setChecked ( true );

	// The comment text box:
	m_comment = new QTextEdit ( box );
	m_comment->setTabChangesFocus ( true );
//FIXME 1.5	m_comment->setTextFormat ( QTextEdit::PlainText );
	m_comment->setText ( initialComment );
	boxLayout->addWidget(m_comment);

	QHBoxLayout *layout = new QHBoxLayout();   //DJ
	pageLayout->addLayout(layout);   //DJ
	m_showButtons = new QCheckBox ( i18n ( "Show comment buttons below &window titlebars" ), page );
	m_showButtons->setChecked ( m_likeBack->userWantsToShowBar() );
	//DJ start
	//pageLayout->addWidget ( m_showButtons );
	//connect ( m_showButtons, SIGNAL ( stateChanged ( int ) ), this, SLOT ( changeButtonBarVisible() ) );
	connect( m_showButtons, SIGNAL(toggled(bool)), SLOT(slotShowToggled(bool)) );
	layout->addWidget(m_showButtons);
	m_saveButton = new QPushButton(i18n("Save Setting"), page);
	m_saveButton->setEnabled(false);
	connect( m_saveButton, SIGNAL(clicked()), SLOT(slotSaveSetting()) );
	layout->addWidget(m_saveButton);
	layout->addStretch();
	//DJ end

	setButtonGuiItem ( KDialog::Ok, KGuiItem ( i18n ( "&Send Comment" ), "mail-send" ) );
	enableButtonOk ( false );
	connect ( m_comment, SIGNAL ( textChanged() ), this, SLOT ( commentChanged() ) );

	setButtonGuiItem ( Default, KGuiItem ( i18n ( "&Email Address..." ), "mail" ) );
	connect(this, SIGNAL(defaultClicked()), SLOT(slotDefault()));

	resize ( QSize ( kapp->desktop()->width() * 1 / 2, kapp->desktop()->height() * 3 / 5 ).expandedTo ( sizeHint() ) );

	QAction *sendShortcut = new QAction ( this );
	sendShortcut->setShortcut ( QString ( "Ctrl+Return" ) );
	connect ( sendShortcut, SIGNAL ( activated() ), button ( KDialog::Ok ), SLOT ( animateClick() ) );

	setMainWidget ( page );
}

/*FIXME LikeBackDialog::~LikeBackDialog()
{}*/

QString LikeBackDialog::introductionText()
{
	QString text = "<p>" + i18n ( "Please provide a brief description of your opinion of %1.", m_likeBack->aboutData()->programName() ) + " ";

	QString languagesMessage = "";
	if ( !m_likeBack->acceptedLocales().isEmpty() && !m_likeBack->acceptedLanguagesMessage().isEmpty() )
	{
		languagesMessage = m_likeBack->acceptedLanguagesMessage();
		QStringList locales = m_likeBack->acceptedLocales();
		for ( QStringList::Iterator it = locales.begin(); it != locales.end(); ++it )
		{
			QString locale = *it;
			if ( KGlobal::locale()->language().startsWith ( locale ) )
				languagesMessage = "";
		}
	}
	else
	{
		//DJ start
		if ( !KGlobal::locale()->language().startsWith ( "en" )
		&&   !KGlobal::locale()->language().startsWith ( "fr" ) )
			languagesMessage = i18n ( "Please write in English or French." );
		//DJ end
	}

	if ( !languagesMessage.isEmpty() )
		// TODO: Replace the URL with a localized one:
		text += languagesMessage + " " +
		        i18n ( "You may be able to use an <a href=\"%1\">online translation tool</a>.",
		        "http://www.google.com/language_tools?hl=" + KGlobal::locale()->language() )
		        + " ";

	// If both "I Like" and "I Dislike" buttons are shown and one is clicked:
	if ( ( m_likeBack->buttons() & LikeBack::Like ) && ( m_likeBack->buttons() & LikeBack::Dislike ) )
		text += i18n ( "To make the comments you send more useful in improving KAlarm, try to send the same amount of positive and negative comments." ) + " ";    //DJ

	if ( ! ( m_likeBack->buttons() & LikeBack::Feature ) )
		text += i18n ( "<p>Please do <b>not</b> use this form to report bugs or ask for new features; use the Report Bug option in the Help menu instead.");   //DJ

	return text;
}

void LikeBackDialog::polish()
{
//FIXME 1.5	KDialog::polish();
	m_comment->setFocus();
}

void LikeBackDialog::slotDefault()
{
	m_likeBack->askEmailAddress();
}

void LikeBackDialog::slotOk()
{
	m_likeBack->setUserWantsToShowBar ( m_showButtons->isChecked() );   //DJ
	send();
}

//DJ start
void LikeBackDialog::slotShowToggled(bool on)
{
	m_saveButton->setEnabled(on != m_originalShow);
}

void LikeBackDialog::slotSaveSetting()
{
	m_originalShow = m_showButtons->isChecked();
	m_likeBack->setUserWantsToShowBar(m_originalShow);
	m_saveButton->setEnabled(false);
}
//DJ end

void LikeBackDialog::commentChanged()
{
	QPushButton *sendButton = button ( KDialog::Ok );
	sendButton->setEnabled ( !m_comment->toPlainText().isEmpty() );
}

void LikeBackDialog::send()
{
	QString emailAddress = m_likeBack->emailAddress();

	int reason = m_group->checkedId ();
	QString type = ( reason == LikeBack::Like ? "Like" : ( reason == LikeBack::Dislike ? "Dislike" : ( reason == LikeBack::Bug ? "Bug" : "Feature" ) ) );
	//DJ start
	QString version = m_likeBack->aboutData()->version();
	if (!m_likeBack->subVersion().isEmpty())
		version += "-" + m_likeBack->subVersion();
	//DJ end
	QString data =
	    "protocol=" + QUrl::toPercentEncoding ( "1.0" )                              + '&' +
	    "type="     + QUrl::toPercentEncoding ( type )                               + '&' +
	    "version="  + QUrl::toPercentEncoding ( version )                            + '&' +    //DJ
	    "locale="   + QUrl::toPercentEncoding ( KGlobal::locale()->language() )      + '&' +
	    "window="   + QUrl::toPercentEncoding ( m_windowPath )                       + '&' +
	    "context="  + QUrl::toPercentEncoding ( m_context )                          + '&' +
	    "comment="  + QUrl::toPercentEncoding ( m_comment->toPlainText() )                  + '&' +
	    "email="    + QUrl::toPercentEncoding ( emailAddress );
	QHttp *http = new QHttp ( m_likeBack->hostName(), m_likeBack->hostPort() );

	QString debugStr;
	QTextStream ( &debugStr ) << "http://" << m_likeBack->hostName() << ":" << m_likeBack->hostPort() << m_likeBack->remotePath();
	qDebug() <<debugStr;
	qDebug() <<data;
	connect ( http, SIGNAL ( requestFinished ( int, bool ) ), this, SLOT ( requestFinished ( int, bool ) ) );

	QHttpRequestHeader header ( "POST", m_likeBack->remotePath() );
	header.setValue ( "Host", m_likeBack->hostName() );
	header.setValue ( "Content-Type", "application/x-www-form-urlencoded" );
	http->setHost ( m_likeBack->hostName() );
	http->request ( header, data.toUtf8() );

	m_comment->setEnabled ( false );
}

void LikeBackDialog::requestFinished ( int /*id*/, bool error )
{
	// TODO: Save to file if error (connection not present at the moment)
	m_comment->setEnabled ( true );
	m_likeBack->disableBar();
	if ( error )
	{
		KMessageBox::error ( this, i18n ( "<p>Error while trying to send the report.</p><p>Please retry later.</p>" ), i18n ( "Transfer Error" ) );
	}
	else
	{
		KMessageBox::information (
		    this,
		    i18n ( "<p>Your comment has been sent successfully. It will help improve KAlarm.</p><p>Thanks for your time.</p>" ),    //DJ
		    i18n ( "Comment Sent" )
		);
		close();
	}
	m_likeBack->enableBar();

	KDialog::slotButtonClicked ( KDialog::Ok );
}

//#include "likeback_private.moc.cpp"
//#include "likeback.moc"
